import sys, os
from .html_to_telegraph import TelegraphPoster


if len(sys.argv) > 1:
    t = TelegraphPoster()
    params = sys.argv[1:]
    t.create_api_token(*params)

    with open('.env', 'a',) as f:
        f.write('\n\nTELEGRAPH_ACCESS_TOKEN=%s' % t.access_token)

    print("Access Token: {access_token}\n"
          "Account Name: {short_name}\n"
          "Author name: {author_name}\n"
          "Author url: {author_url}\n"
          "Authorization url: {auth_url}\n"
          "File .env generated".format(**t.account)
    )
else:
    print('Usage: '
          'Account Name (required)'
          'Author name (optional)'
          'Author url (optional)'
    )



import sys, os
from html_telegraph_poster_v2.async_poster.html_to_telegraph import AsyncTelegraphPoster


# if len(sys.argv) > 1:
#     t = AsyncTelegraphPoster()
#     params = sys.argv[1:]
#     await t.create_api_token(*params)
#
#     with open('.env', 'a',) as f:
#         f.write('\n\nTELEGRAPH_ACCESS_TOKEN=%s' % t.access_token)
#
#     print("Access Token: {access_token}\n"
#           "Account Name: {short_name}\n"
#           "Author name: {author_name}\n"
#           "Author url: {author_url}\n"
#           "Authorization url: {auth_url}\n"
#           "File .env generated".format(**t.account)
#     )
# else:
#     print('Usage: '
#           'Account Name (required)'
#           'Author name (optional)'
#           'Author url (optional)'
#     )



# coding=utf8


class Error(Exception):
    pass


class TitleRequiredError(Error):
    pass


class TextRequiredError(Error):
    pass


class APITokenRequiredError(Error):
    pass


class GetImageRequestError(Error):
    pass


class ImageUploadHTTPError(Error):
    pass


class FileTypeNotSupported(Error):
    pass


class TelegraphUnknownError(Error):
    pass


class TelegraphPageSaveFailed(Error):
    # reason is unknown
    pass


class TelegraphContentTooBigError(Error):
    def __init__(self, message):
        message += ". Max size is 64kb including markup"
        super(Error, TelegraphError).__init__(self, message)


class TelegraphFloodWaitError(Error):
    def __init__(self, message):
        super(Error, TelegraphError).__init__(self, message)
        self.FLOOD_WAIT_IN_SECONDS = int(message.split('FLOOD_WAIT_')[1])


class TelegraphError(Error):
    def __init__(self, message):
        if 'Unknown error' in message:
            raise TelegraphUnknownError(message)
        elif 'Content is too big' in message:
            raise TelegraphContentTooBigError(message)
        elif 'FLOOD_WAIT_' in message:
            raise TelegraphFloodWaitError(message)
        elif 'PAGE_SAVE_FAILED' in message:
            raise TelegraphPageSaveFailed(message)
        else:
            super(Error, TelegraphError).__init__(self, message)



import base64
import os.path

import httpx

from html_telegraph_poster_v2.async_poster.image_upload.image_uploader import ImageUploader
from html_telegraph_poster_v2.config import GITHUB_REPO, GITHUB_BRANCH, GITHUB_PATH, GITHUB_COMMIT_MESSAGE
from html_telegraph_poster_v2.utils.logger import logger
from html_telegraph_poster_v2.utils.parse import check_url_is_local


class GithubUploader(ImageUploader):
    def __init__(self, token: str):
        self.token = token

    async def upload_file(self, file_url: str, repo: str = GITHUB_REPO, branch: str = GITHUB_BRANCH,
                          path: str = GITHUB_PATH, message: str = GITHUB_COMMIT_MESSAGE) -> str:
        url = f"https://api.github.com/repos/{repo}/contents/{path}"
        headers = {
            "Authorization": f"token {self.token}",
            "Accept": "application/vnd.github.v3+json",
        }
        async with httpx.AsyncClient() as client:
            if check_url_is_local(file_url):
                if not os.path.exists(file_url):
                    logger.error(f"File {file_url} does not exist")
                    raise FileNotFoundError(f"File {file_url} does not exist")
                with open(file_url, "rb") as f:
                    content = base64.b64encode(f.read()).decode("utf-8")
            else:
                response = await client.get(file_url)
                response.raise_for_status()
                content = base64.b64encode(response.content).decode("utf-8")
            data = {
                "message": message,
                "content": content,
                "branch": branch,
            }
            response = await client.put(
                url=url,
                headers=headers,
                json=data,
            )
            if response.status_code == 201:
                logger.info(f"Uploaded {file_url} to {repo}/{path}")
                return response.json()["content"]["download_url"]
            else:
                logger.error(f"Failed to upload {file_url} to {repo}/{path}, {response.text}")
                return ""



import asyncio
import os
import uuid
from datetime import datetime
from urllib.parse import urlparse, quote

import aiofiles.os
from pathlib import Path

import aioboto3
import httpx
from botocore.exceptions import ClientError

from html_telegraph_poster_v2.utils.logger import logger
from .image_uploader import ImageUploader
from html_telegraph_poster_v2.config import AWS_S3_BUCKET_NAME, AWS_REGION_NAME, AWS_DOMAIN_HOST, AWS_ACCESS_KEY_ID, \
    AWS_SECRET_ACCESS_KEY, AWS_S3_OBJECT_KEY
from ...utils.parse import check_url_is_local

image_url_host = (
    AWS_DOMAIN_HOST
    if AWS_DOMAIN_HOST
    else f"{AWS_S3_BUCKET_NAME}.s3.{AWS_REGION_NAME}.amazonaws.com"
)


class AWSUploader(ImageUploader):

    def __init__(self, access_key_id: str = AWS_ACCESS_KEY_ID, secret_access_key: str = AWS_SECRET_ACCESS_KEY,
                 region_name: str = AWS_REGION_NAME):
        self.access_key_id = access_key_id
        self.secret_access_key = secret_access_key
        self.region_name = region_name

    async def upload_file(self, file_url: str, bucket_name: str = AWS_S3_BUCKET_NAME,
                          object_key: str = AWS_S3_OBJECT_KEY) -> str:
        # the object_key should be the directory path. We will append the file name to it
        if object_key.endswith("/"):
            object_key += Path(file_url).name
        else:
            object_key += "/" + Path(file_url).name

        if check_url_is_local(file_url):
            if not os.path.exists(file_url):
                raise FileNotFoundError(f"Local file '{file_url}' not found")
            with open(file_url, "rb") as file:
                content = file.read()
        else:
            async with httpx.AsyncClient() as client:
                response = await client.get(file_url)
                response.raise_for_status()
                if response.status_code == 200:
                    content = response.content
                else:
                    logger.error(f"Failed to download {file_url}")
                    raise FileNotFoundError(f"Failed to download {file_url}")
        async with aioboto3.Session().client(
                "s3",
                region_name=self.region_name,
                aws_access_key_id=self.access_key_id,
                aws_secret_access_key=self.secret_access_key,
        ) as s3_client:
            try:
                response = await s3_client.put_object(
                    Bucket=bucket_name,
                    Key=object_key,
                    Body=content,
                )
                if response["ResponseMetadata"]["HTTPStatusCode"] == 200:
                    logger.info(f"Uploaded {file_url} to {bucket_name}/{object_key}")
                    return f"https://{image_url_host}/{object_key}"
                else:
                    logger.error(f"Failed to upload {file_url} to {bucket_name}/{object_key}")
                    return ""
            except ClientError as e:
                logger.error(f"Failed to upload {file_url} to {bucket_name}/{object_key}, {e}")
                return ""



import os
from typing import Optional

env = os.environ


def get_bool(value: Optional[str], default: bool = True) -> bool:
    true_values = ("True", "true", "1", "yes", "on")
    false_values = ("False", "false", "0", "no", "off")

    if value is None:
        return default
    value = value.lower()

    if value in true_values:
        return True
    elif value in false_values:
        return False
    else:
        return default


def get_env_bool(env, var_name: Optional[str], default: bool = False):
    """Retrieve environment variable as a boolean."""
    value = env.get(var_name, "").lower()
    return get_bool(value, default)


# General
DEFAULT_USER_AGENT = env.get("DEFAULT_USER_AGENT", "Python_html_telegraph_poster_v2/0.1")
DOWNLOAD_DIR = env.get("DOWNLOAD_DIR", os.path.join(os.getcwd(), "download"))

# Image Uploading
# AWS S3
AWS_ACCESS_KEY_ID = env.get("AWS_ACCESS_KEY_ID", None)
AWS_SECRET_ACCESS_KEY = env.get("AWS_SECRET_ACCESS_KEY", None)
AWS_S3_BUCKET_NAME = env.get("AWS_S3_BUCKET_NAME", None)
AWS_REGION_NAME = env.get("AWS_REGION_NAME", None)
AWS_S3_OBJECT_KEY = env.get("AWS_OBJECT_KEY", None)
AWS_DOMAIN_HOST = env.get("AWS_DOMAIN_HOST", None)
# GitHub
GITHUB_TOKEN = env.get("GITHUB_TOKEN", None)
GITHUB_REPO = env.get("GITHUB_REPO", None)
GITHUB_BRANCH = env.get("GITHUB_BRANCH", None)
GITHUB_PATH = env.get("GITHUB_PATH", None)
GITHUB_COMMIT_MESSAGE = env.get("GITHUB_COMMIT_MESSAGE", None)


# coding=utf8
from typing import Optional, Union
from urllib.parse import urlparse, urljoin
import logging

from .image_upload import uploader_list
from .image_upload.image_uploader import ImageUploader
from .upload_images import upload_image
from .converter import _fragments_from_string
import lxml.html
import concurrent.futures

from html_telegraph_poster_v2.utils.logger import logger
from html_telegraph_poster_v2.config import (
    AWS_ACCESS_KEY_ID,
    AWS_SECRET_ACCESS_KEY,
    AWS_S3_BUCKET_NAME,
)

LOG = logging.getLogger(__name__)


class DocumentPreprocessor:
    def __init__(self,
                 input_document: Union[str | lxml.html.HtmlMixin],
                 url: str = None,
                 image_uploader: str = None):
        self.url = url
        self.input_document = input_document
        self.parsed_document = self._parse_document()
        self.image_uploader_type: str = image_uploader
        if image_uploader:
            self.image_uploader: ImageUploader = uploader_list.get(image_uploader)()
        else:
            self.image_uploader = None

    def get_processed_html(self):
        return lxml.html.tostring(self.parsed_document, encoding="unicode")

    @staticmethod
    async def _upload_image(url, image_uploader: ImageUploader, **kwargs):
        new_image_url = None
        try:
            new_image_url = await image_uploader.upload_image(url, **kwargs)
        except Exception:
            logger.error(f"Could not upload image {url}")

        return new_image_url

    async def upload_all_images(self, base_url=None, **kwargs):
        self._make_links_absolute(base_url)
        images = self.parsed_document.xpath(
            './/img[@src][not(contains(@src, "//telegra.ph/file/")) and'
            ' not(contains(@src, "//graph.org/file/"))]'
        )

        for image in images:
            logger.debug(f"Uploading image {image.attrib.get('src')}")
            await DocumentPreprocessor._upload_and_replace_url(image, url=self.url, image_uploader=self.image_uploader, **kwargs)

    @staticmethod
    async def _upload_and_replace_url(image_element, url: str = None, image_uploader: ImageUploader = None):
        old_image_url = image_element.attrib.get("src")
        new_image_url = await DocumentPreprocessor._upload_image(old_image_url, image_uploader)
        if new_image_url:
            image_element.attrib.update({"src": new_image_url})

    def _parse_document(self):
        if isinstance(self.input_document, str):
            fragments = _fragments_from_string(self.input_document)
            document = fragments[0].xpath("/*")[0] if len(fragments) else None
        elif isinstance(self.input_document, lxml.html.HtmlMixin):
            document = self.input_document.xpath("/*")[0]
        else:
            raise TypeError(
                "DocumentPreprocessor accepts only html string or lxml document object"
            )

        return document

    def _make_links_absolute(self, base_url=None):
        body = self.parsed_document.body
        output_base = None
        document_base_url = self.parsed_document.base

        if base_url:
            urlformat = urlparse(base_url)
            url_without_path = urlformat.scheme + "://" + urlformat.netloc
            output_base = url_without_path + urlformat.path
        elif document_base_url:
            if urlparse(document_base_url).netloc:
                output_base = document_base_url

        if output_base is None:
            # no base_url was passed, document_base_url is missing
            LOG.warning("Relative image/link urls were removed from the document")

        def link_replace(href):
            try:
                if output_base is None and not urlparse(href).netloc:
                    url = None
                else:
                    url = urljoin(output_base, href)
                return url
            except ValueError:
                return None

        body.rewrite_links(link_repl_func=link_replace, base_href=output_base)



from .html_to_telegraph import upload_to_telegraph, AsyncTelegraphPoster
from .upload_images import upload_image
from .utils import DocumentPreprocessor
import logging

logging.getLogger(__name__).addHandler(logging.NullHandler())



import mimetypes
import re
import requests

from io import BytesIO
base_url = 'http://telegra.ph'
save_url = 'https://edit.telegra.ph/save'
upload_file_url = 'https://telegra.ph/upload'


class Error(Exception):
    pass


class GetImageRequestError(Error):
    pass


class ImageUploadHTTPError(Error):
    pass


class FileTypeNotSupported(Error):
    pass


def _check_mimetypes(type):
    return type in ('image/jpeg', 'image/png', 'image/gif', 'video/mp4')


def _get_mimetype_from_response_headers(headers):
    types = re.split(r'[;,]', headers['Content-Type'])
    if len(types):
        ext = mimetypes.guess_extension(types[0], strict=False)
        if ext:
            return mimetypes.types_map.get(ext, mimetypes.common_types.get(ext, ''))
    return ''


def upload_image(
        file_name_or_url,
        user_agent='Python_telegraph_poster/0.1',
        return_json=False,
        get_timeout=(10.0, 10.0),
        upload_timeout=(7.0, 7.0)
    ):

    if hasattr(file_name_or_url, 'read') and hasattr(file_name_or_url, 'name'):
        img = file_name_or_url
        img_content_type = mimetypes.guess_type(file_name_or_url.name)[0]
    elif re.match(r'^https?://', file_name_or_url, flags=re.IGNORECASE):
        try:
            img = requests.get(file_name_or_url, headers={'User-Agent': user_agent}, timeout=get_timeout)
        except:
            raise GetImageRequestError('Url request failed')

        if img.status_code != 200 or 'Content-Type' not in img.headers:
            raise GetImageRequestError('Url request failed')

        img_content_type = _get_mimetype_from_response_headers(img.headers)
        img = BytesIO(img.content)

    else:
        img_content_type = mimetypes.guess_type(file_name_or_url)[0]
        img = open(file_name_or_url, 'rb').read()

    # simple filecheck, based on file extension
    if not _check_mimetypes(img_content_type):
        raise FileTypeNotSupported('The "%s" filetype is not supported' % img_content_type)

    headers = {
        'X-Requested-With': 'XMLHttpRequest',
        'Accept': 'application/json, text/javascript, */*; q=0.01',
        'Referer': base_url + '/',
        'User-Agent': user_agent
    }

    files = {
        'file': ('blob', img, img_content_type)
    }
    try:
        json_response = requests.post(upload_file_url, timeout=upload_timeout, files=files, headers=headers)
    except requests.exceptions.ReadTimeout:
        raise ImageUploadHTTPError('Request timeout')

    if json_response.status_code == requests.codes.ok and json_response.content:
        json_response = json_response.json()
        if return_json:
            return json_response
        elif type(json_response) is list and len(json_response):
            return 'src' in json_response[0] and base_url + json_response[0]['src'] or ''
        elif type(json_response) is dict:
            if json_response.get('error') == 'File type invalid':
                raise FileTypeNotSupported('This file is unsupported')
            else:
                return str(json_response)
    else:
        raise Exception('Error while uploading the image')



# encoding=utf8
import json
import os
import requests
from requests_toolbelt import MultipartEncoder
from .errors import *
from .converter import convert_html_to_telegraph_format, convert_json_to_html, OutputFormat

base_url = 'http://telegra.ph'
save_url = 'https://edit.telegra.ph/save'
api_url = 'https://api.telegra.ph'
default_user_agent = 'Python_telegraph_poster/0.1'


def _upload(title, author, text,
            author_url='', tph_uuid=None, page_id=None, user_agent=default_user_agent, convert_html=True,
            clean_html=True, telegraph_base_url=base_url):

    if not title:
        raise TitleRequiredError('Title is required')
    if not text:
        raise TextRequiredError('Text is required')

    content = convert_html_to_telegraph_format(text, clean_html) if convert_html else text
    cookies = dict(tph_uuid=tph_uuid) if tph_uuid and page_id else None

    fields = {
        'Data': ('content.html', content, 'plain/text'),
        'title': title,
        'author': author,
        'author_url': author_url,
        'page_id': page_id or '0',
        'save_hash': ''
    }

    m = MultipartEncoder(fields, boundary='TelegraPhBoundary21')

    headers = {
        'Content-Type': m.content_type,
        'Accept': 'application/json, text/javascript, */*; q=0.01',
        'User-Agent': user_agent,
        'Origin': telegraph_base_url
    }
    with requests.Session() as r:
        r.mount('https://', requests.adapters.HTTPAdapter(max_retries=3))

        response = r.post(save_url, timeout=4, headers=headers, cookies=cookies, data=m.to_string())
        result = json.loads(response.text)
        if 'path' in result:
            result['tph_uuid'] = response.cookies.get('tph_uuid') or tph_uuid
            result['url'] = telegraph_base_url + '/' + result['path']
            return result
        else:
            error_msg = result['error'] if 'error' in result else ''
            raise TelegraphError(error_msg)


def _prepare_page_upload_params(params):
    # significantly reduce size of request body
    return json.dumps(params, ensure_ascii=False, separators=(',', ':')).encode('utf-8')


def _upload_via_api(title, author, text, author_url='', access_token=None, user_agent=default_user_agent,
                    convert_html=True,  clean_html=True, path=None, telegraph_api_url=api_url):

    if not title:
        raise TitleRequiredError('Title is required')
    if not text:
        raise TextRequiredError('Text is required')
    if not access_token:
        raise APITokenRequiredError('API token is required')
    if not author:
        author = ''  # author is optional
    if not author_url:
        author_url = ''  # author_url is optional

    content = convert_html_to_telegraph_format(text, clean_html, output_format=OutputFormat.PYTHON_LIST) if convert_html else text
    method = '/createPage' if not path else '/editPage'

    params = {
        'access_token': access_token,
        'title': title[:256],
        'author_name': author[:128],
        'author_url': author_url[:512],
        'content': content,
    }
    request_headers = {
        'User-Agent': user_agent,
        'Content-Type': 'application/json'
    }
    if path:
        params.update({'path': path})

    resp = requests.post(telegraph_api_url + method, data=_prepare_page_upload_params(params), headers=request_headers).json()
    if resp['ok'] is True:
        return resp.get('result')
    else:
        error_msg = resp['error'] if 'error' in resp else ''
        raise TelegraphError(error_msg)


def create_api_token(short_name, author_name=None, author_url=None, user_agent=default_user_agent):
    params = {
        'short_name': short_name,
    }
    if author_name:
        params.update({'author_name': author_name})
    if author_url:
        params.update({'author_url': author_url})

    resp = requests.get(api_url+'/createAccount', params, headers={'User-Agent': user_agent})
    json_data = resp.json()
    return json_data['result']


def upload_to_telegraph(title, author, text, author_url='', tph_uuid=None, page_id=None, user_agent=default_user_agent):
    return _upload(title, author, text, author_url, tph_uuid, page_id, user_agent)


class TelegraphPoster(object):
    def __init__(self, tph_uuid=None, page_id=None, user_agent=default_user_agent, clean_html=True, convert_html=True,
                 use_api=False, access_token=None, telegraph_api_url=api_url, telegraph_base_url=base_url):
        self.title = None
        self.author = None
        self.author_url = None
        self.text = None
        self.path = None
        self.tph_uuid = tph_uuid
        self.page_id = page_id
        self.user_agent = user_agent
        self.clean_html = clean_html
        self.convert_html = convert_html
        self.access_token = access_token or os.getenv('TELEGRAPH_ACCESS_TOKEN', None)
        self.account = None
        self.use_api = use_api
        self.telegraph_api_url = telegraph_api_url
        self.telegraph_base_url = telegraph_base_url
        if self.access_token:
            # use api anyway
            self.use_api = True

    def _api_request(self, method, params=None):
        params = params or {}
        if self.access_token:
            params['access_token'] = self.access_token
        resp = requests.get(self.telegraph_api_url + '/' + method, params, headers={'User-Agent': self.user_agent})
        return resp.json()

    def post(self, title, author, text, author_url=''):
        self.path = None
        self.title = title
        self.author = author
        self.author_url = author_url
        self.text = text
        result = self.edit()
        if not self.use_api:
            self.tph_uuid = result['tph_uuid']
            self.page_id = result['page_id']
        return result

    def edit(self, title=None, author=None, text=None, author_url='', path=None):
        params = {
            'title': title or self.title,
            'author': author or self.author,
            'text': text or self.text,
            'author_url': author_url or self.author_url,
            'user_agent': self.user_agent,
            'clean_html': self.clean_html,
            'convert_html': self.convert_html
        }
        if self.use_api:
            params['telegraph_api_url'] = self.telegraph_api_url
            result = _upload_via_api(access_token=self.access_token, path=path or self.path, **params)
            self.path = result['path']
            return result
        else:
            return _upload(
                tph_uuid=self.tph_uuid,
                page_id=self.page_id,
                **params
            )

    def get_account_info(self, fields=None):
        """
        Use this method to get information about a Telegraph account.
        :param fields: (Array of String, default = ['short_name','author_name','author_url'])
        List of account fields to return. Available fields: short_name, author_name, author_url, auth_url, page_count.
        :return: Returns an Account object on success.
        """
        if not self.access_token:
            raise Exception('Access token is required')

        return self._api_request('getAccountInfo', {
            'fields': json.dumps(fields) if fields else ''
        }).get('result')

    def edit_account_info(self, short_name, author_name='', author_url=''):
        """
            Use this method to update information about a Telegraph account.
            Pass only the parameters that you want to edit
        :param short_name: (String, 1-32 characters) New account name.
        :param author_name: (String, 0-128 characters) New default author name used when creating new articles.
        :param author_url: (String, 0-512 characters) New default profile link, opened when users click on the
            author's name below the title.
            Can be any link, not necessarily to a Telegram profile or channel.
        :return:  Account object with the default fields.
        """
        if not self.access_token:
            raise Exception('Access token is required')
        params = {
            'short_name': short_name
        }
        if author_name:
            params['author_name'] = author_name
        if author_url:
            params['author_url'] = author_url
        return self._api_request('editAccountInfo', params).get('result')

    def get_page(self, path, return_content=False):
        """
        Use this method to get a Telegraph page. Returns a Page object on success.
        :param path:  (String) Required. Path to the Telegraph page (in the format Title-12-31, i.e.
            everything that comes after http://telegra.ph/).
        :param return_content: (Boolean, default = false) If true, content field will be returned in Page object.
        :return: Returns a Page object on success
        """
        json_response = self._api_request('getPage', {
            'path': path,
            'return_content': return_content
        })
        if return_content:
            json_response['result']['html'] = convert_json_to_html(json_response['result']['content'], self.telegraph_base_url)
        return json_response.get('result')

    def get_page_list(self, offset=0, limit=50):
        """
            Use this method to get a list of pages belonging to a Telegraph account.
        :param offset: Sequential number of the first page to be returned.
        :param limit: Limits the number of pages to be retrieved.
        :return: PageList object, sorted by most recently created pages first.
        """
        json_response = self._api_request('getPageList', {
            'offset': offset,
            'limit': limit
        })
        return json_response.get('result')

    def get_views(self, path, year=None, month=None, day=None, hour=None):
        """
            Use this method to get the number of views for a Telegraph article.
        :param path: Required. Path to the Telegraph page (in the format Title-12-31, where 12 is the month and 31 the
            day the article was first published).
        :param year: Required if month is passed. If passed, the number of page views for the requested year will be
            returned.
        :param month: Required if day is passed. If passed, the number of page views for the requested month will be
            returned.
        :param day: Required if hour is passed. If passed, the number of page views for the requested day will be
            returned.
        :param hour: If passed, the number of page views for the requested hour will be returned.
        :return: Returns a PageViews object on success. By default, the total number of page views will be returned.
        """
        return self._api_request('getViews', {
            'path': path,
            'year': year,
            'month': month,
            'day': day,
            'hour': hour
        }).get('result')

    def create_api_token(self, short_name, author_name=None, author_url=None):
        """
            Use this method to create a new Telegraph account.
            Most users only need one account, but this can be useful for channel administrators who would like to keep
            individual author names and profile links for each of their channels.
        :param short_name: Account name, helps users with several accounts remember which they are currently using.
            Displayed to the user above the "Edit/Publish" button on Telegra.ph, other users don't see this name.
        :param author_name: Default author name used when creating new articles.
        :param author_url: Default profile link, opened when users click on the author's name below the title.
            Can be any link, not necessarily to a Telegram profile or channel.
        :return: Account object with the regular fields and an additional access_token field.
        """
        token_data = create_api_token(short_name, author_name, author_url, self.user_agent)
        self.use_api = True
        self.account = token_data
        self.access_token = token_data['access_token']
        return token_data

    def revoke_access_token(self):
        """
            Use this method to revoke access_token and generate a new one, for example, if the user would like to reset
            all connected sessions, or you have reasons to believe the token was compromised
        :return: Account object with new access_token and auth_url fields.
        """
        if not self.access_token:
            raise Exception('Access token is required')

        json_response = self._api_request('revokeAccessToken')
        if json_response['ok'] is True:
            self.access_token = json_response['result']['access_token']

        return json_response['result']

    def create_page(self, *args, **kwargs):
        """
            Shortcut method for post()
        """
        return self.post(*args, **kwargs)

    def edit_page(self, *args, **kwargs):
        """
            Shortcut method for edit()
        """
        return self.edit(*args, **kwargs)

    def create_account(self, *args, **kwargs):
        """
            Shortcut method for create_api_token()
        """
        return self.create_api_token(*args, **kwargs)
